-- Title       : Driver 625_lignes
-- Author      : Frdric Cabanes
-- File        : Driver_625_lignes.vhd
-- Generated   : Dimanche 6 Fvrier 15:20:52 2011
-- Description : Ralisation d'un driver pour le convertisseur vido 625 vers 819 lignes en noir et blanc.
-- Gnration des divers signaux composant la commande du TDA8708A, des fifos et du module de gestion du timing 819 lignes
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_unsigned.all;
use IEEE.STD_LOGIC_ARITH.ALL;

entity driver_625_lignes is

	 port( mclk : in STD_LOGIC; -- signal d'horloge interne principal cadenc  50MHz
		raz : in STD_LOGIC;-- remise  zro externe par circuit RC
		hsync : in STD_LOGIC;-- signal de synchronisation horizontale venant du LM1881
		vsync : in STD_LOGIC;-- signal de synchronisation verticale venant du LM1881
		BURST_BKG : in STD_LOGIC;-- signal de burst blanking venant du LM1881
		GATE_A : out STD_LOGIC;-- signal de commande du TDA8708A pour mode 2
		GATE_B : out STD_LOGIC;-- signal de commande du TDA8708A pour mode 2
		Sel_TDA8708A : out STD_LOGIC;-- signal d'activation de l'entre vido du TDA8708A
		FIFO_WE : out STD_LOGIC;-- signal d'activation en criture des fifos
		pixel_clk : out STD_LOGIC;-- horloge pixel pour FIFOs
		clk_TDA8708 : out STD_LOGIC;-- horloge pixel pour TDA8708A > frquence d'chantillonnage
		read_process_en : out std_logic; -- signal de dmarrage du process 819 lignes	
		RST_FIFO : out STD_LOGIC-- signal de remise  zro des FIFOs actif  chaque trame pour cpt_fifo=11
		 );
end driver_625_lignes;

architecture driver_625_lignes of driver_625_lignes is

constant hpixels: std_logic_vector(10 downto 0) :="11001000000";
		--nombre de cycles horloge pour atteindre la fin de ligne = 1600 : 40 ns * 1600= 64us
constant vlines: std_logic_vector(9 downto 0) :="1001110001";
		--nombre maximal de lignes dans l'cran = 625
constant hbp: std_logic_vector(7 downto 0) :="11111010";
		--Palier arrire horizontal = 250 correspond  dbut d'image  10 S
constant hfp: std_logic_vector(10 downto 0) :="11000001110";
		--Palier avant horizontal = 1550 correspond  la fin de l'image  62 S
constant vbpf1: std_logic_vector(4 downto 0) :="10111";
		--Palier arrire vertical frame N1 = 23, dbut de la premire ligne visible du champs N1
constant vfpf1: std_logic_vector(8 downto 0) :="100110110";
		--Palier avant vertical frame N1 = 311, dernire ligne visible du champs N1
constant vbpf2: std_logic_vector(8 downto 0) :="101010000";
		--Palier arrire vertical frame N2 = 336, premire ligne visible du champs N2
constant vfpf2: std_logic_vector(9 downto 0) :="1001101111";
		--Palier avant vertical frame N2 = 623, dernire ligne visible du champs N1 et de l'image
constant ligne_620: std_logic_vector(10 downto 0) :="01001101100";	--01001101100
		--Dtection de la ligne 620 qui permet le dmarrage du bloc 819 lignes
constant ligne_619: std_logic_vector(10 downto 0) :="01001101011";	
		--Dtection de la ligne 619 qui permet le reset des pointeurs des fifos pour trame 1
constant ligne_300: std_logic_vector(10 downto 0) :="00100101100";	
		--Dtection de la ligne 300 qui permet le reset des pointeurs des fifos pour trame 2
signal hcs, vcs: std_logic_vector(10 downto 0);
		-- ce sont les signaux compteurs courants horizontaux et verticaux en interne
signal vsenable: std_logic;
		-- signal interne pour l'activation du compteur vertical
signal hordiv: std_logic;
		-- signal interne d'horloge divise
signal process_819: std_logic;
		-- signal de dbut du dmarrage du process 819 lignes
signal RST_FIFO_INT: std_logic;
		-- signal interne de remise  zro des fifo 
signal sig_pixel_clk: std_logic;
		-- signal interne d'horloge pixel
signal raz_int: std_logic;
		-- signal interne de remise  zro des fifo 
signal clkdiv: std_logic_vector(1 downto 0);
		-- registre de decalge pour division de la frquence d'horloge
signal vsync_inverse: std_logic;
		-- signal invers
signal vsync_ret: std_logic;
		-- signal vsync retarde
signal vsync_reset: std_logic;
		-- signal reset vsync

begin

hordiv <= clkdiv(0); -- Diviseur de frequence d'horloge  25Mhz
sig_pixel_clk <= clkdiv(1); -- Diviseur de frequence d'horloge  12.5Mhz

vsync_reset <= vsync_inverse and vsync_ret;

raz_int <= raz or vsync_reset;-- raz_int est gal  raz ou vsync_rst

GATE_A <= not hsync;-- inversion de hsync pour commande du mode 2 du TDA8708A
GATE_B <= not BURST_BKG;--inversion de BURST_BKG pour commande du mode 2 du TDA8708A
vsync_inverse <= not vsync;

pixel_clk <= sig_pixel_clk;
clk_TDA8708 <= sig_pixel_clk;


process(mclk, raz)
begin
		if raz ='1' then
		vsync_ret <= '0';
		elsif rising_edge (mclk) then
		vsync_ret <= vsync;
		end if;
end process;		
                              
process(mclk, raz)-- Compteur pour diviser la frquence d'horloge
begin
		if raz = '1' then 
		clkdiv <= "00";
		elsif mclk'event and mclk = '1' then
		clkdiv <= clkdiv + 1;
		end if;
end process;

sel_TDA8708A <= '1' when ((((hcs <= hfp) and (hcs >= hbp)) and ((vcs <= vfpf1) and (vcs >= vbpf1))) or (((hcs <= hfp) and (hcs >= hbp)) and ((vcs <= vfpf2) and (vcs >= vbpf2)))) else '0'; 
-- l'entre 0 du TDA est slectionne pour chantillonnage pendant la vido visible
FIFO_WE <= '1' when ((((hcs <= hfp) and (hcs >= hbp)) and ((vcs <= vfpf1) and (vcs >= vbpf1))) or (((hcs <= hfp) and (hcs >= hbp)) and ((vcs <= vfpf2) and (vcs >= vbpf2)))) else '0'; 
-- Les fifos sont actives en criture pendant la vido visible

process_819 <= '1' when (vcs = ligne_620) else '0';-- Quand VCS atteint la ligne 620
read_process_en <= process_819;

RST_FIFO_INT <= '1' when ((vcs =  ligne_619)  or (vcs = ligne_300)) else '0';-- mise  zro des pointeurs fifos

RST_FIFO <= RST_FIFO_INT;

-- compteur pour le signal de synchro horizontale
	process(hordiv, raz_int)
	begin
		if raz_int ='1' then -- si raz_int = 1 alors mise  zro de hcs
		hcs <= "00000000000"; 
		elsif(hordiv'event and hordiv = '1') then
		if hcs = hpixels - 1 then
	-- Si le compteur hcs a atteint la fin du compteur pixel soit 1600
			hcs <= "00000000000"; -- remise  zro du compteur horizontal
	vsenable <= '1'; -- activation du compteur vertical
		else
			hcs <= hcs + 1; -- incrmentation du compteur horizontal
			vsenable <= '0'; -- laisser vsenable  off
		end if;
	end if;
end process; 

-- compteur pour le signal de synchro verticale
	process(hordiv, raz_int)
	begin
		if raz_int = '1' then
		vcs <="00000000000";
		elsif(hordiv'event and hordiv ='1' and vsenable = '1') then
		-- incrmentation du compteur vertical si activ
			if vcs = vlines - 1 then
		-- si le nombre de lignes max est atteint, remise  zro de vcs
			vcs <= "00000000000";
			else
			vcs <= vcs + 1; --incrmentation du compteur vertical
		end if;
	end if;
end process;

end driver_625_lignes;