-- Title       : mire_819_lignes
-- Author      : Cabanes Frdric
-- File        : mire_819_lignes.vhd
-- Generated   : Thu Mar 18 16:15:52 2010
-- Description : Ralisation d'un convertisseur 819 lignes en noir et blanc.
-- Gnration des divers signaux composant le signal vido.
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_unsigned.all;
use IEEE.STD_LOGIC_ARITH.ALL;

entity mire_819_lignes is
	 port( mclk : in STD_LOGIC; -- signal d'horloge principal cadenc  50MHz
		raz : in STD_LOGIC;-- remise  zro hardware via BTN3
		rd_process_en : in std_logic;-- signal venant du bloc 625
		SYNC_INT : out STD_LOGIC;-- signal de synchronisation verticale
		vidon : out STD_LOGIC;-- signal d'activation de la vido allant  l'interpolateur
		fifo1_rd_en : out std_logic;-- signal d'activation de la fifo1
		fifo2_rd_en : out std_logic;-- signal d'activation de la fifo2
		sel_in : out std_logic_vector(1 downto 0);
		rd_clock_ext : out std_logic;-- horloge pixel 819 lignes pour le TDA8702
		rd_clock_int : out std_logic-- horloge pixel 819 lignes pour le TDA8702
		);
end mire_819_lignes;

architecture mire_819_lignes of mire_819_lignes is

constant hpixels: std_logic_vector(10 downto 0) :="10011000101";
		--nombre maximal de cycles horloge pour atteindre la fin de ligne = 1221
constant vlines: std_logic_vector(10 downto 0) :="01100110011";
		--nombre maximal de lignes dans l'cran = 819
constant hbp: std_logic_vector(10 downto 0) :="00010111011";
		--Palier arrire horizontal = 187
constant hfp: std_logic_vector(10 downto 0) :="10010111000";
		--Palier avant horizontal = 1208
constant vbpf1: std_logic_vector(10 downto 0) :="00000100111";
		--Palier arrire vertical frame N1 = 39
constant vfpf1: std_logic_vector(8 downto 0) :="110010110";
		--Palier avant vertical frame N1 = 406
constant vbpf2: std_logic_vector(8 downto 0) :="110111111";
		--Palier arrire vertical frame N2 = 447
constant vfpf2: std_logic_vector(9 downto 0) :="1100110000";
		--Palier avant vertical frame N2 = 816	
signal hcs, vcs: std_logic_vector(10 downto 0);
		-- ce sont les compteurs courants horizontaux et verticaux
signal vsenable: std_logic;
		-- activation du compteur vertical
signal hordiv: std_logic;
		-- horloge divise
signal cpt_fifo_en: std_logic_vector(1 downto 0);		
		-- signal de compteur d'tat des fifos
signal fifo1_rd_en_in: std_logic;		
		-- signal de compteur d'tat des fifos
signal fifo2_rd_en_in: std_logic;		
		-- signal de compteur d'tat des fifos
signal rd_fifo : std_logic;-- signal activation des fifos en lecture

signal raz_int : std_logic;-- signal de raz interne => raz + rd_process_en

signal clkdiv: std_logic_vector(1 downto 0);
		-- registre de decalge pour division de la frquence d'horloge

begin

hordiv <= clkdiv(0); -- division de la frquence de 50Mhz/2

rd_clock_int <= clkdiv(1);
rd_clock_ext <= clkdiv(1);

raz_int <= raz or rd_process_en;                                


process(mclk, raz)-- Compteur pour diviser la frquence d'horloge
begin
		if raz = '1' then 
		clkdiv <= "00";
		elsif mclk'event and mclk = '1' then
		clkdiv <= clkdiv + 1;
		end if;
end process;

sel_in <= fifo2_rd_en_in & fifo1_rd_en_in;--selection des fifos


-- compteur pour le signal de synchro horizontale
	process(hordiv, raz_int)
	begin
		if raz_int ='1' then
		hcs <= "00000000000"; -- si raz_int = 1 alors mise  zro de hcs
		elsif(hordiv'event and hordiv = '1') then
		if hcs = hpixels - 1 then
	-- Si le compteur hcs a atteint la fin du compteur pixel soit 1221
			hcs <= "00000000000"; -- remise  zro du compteur horizontal
	vsenable <= '1'; -- activation du compteur vertical
		else
			hcs <= hcs + 1; -- incrmentation du compteur horizontal
			vsenable <= '0'; -- laisser vsenable  off
		end if;
	end if;
end process; 

-- compteur pour le signal de synchro verticale
	process(hordiv, raz_int)
	begin
		if raz_int = '1' then
		vcs <="00000000000";
		elsif(hordiv'event and hordiv ='1' and vsenable = '1') then
		-- incrmentation du compteur vertical si activ
			if vcs = vlines - 1 then
		-- si le nombre de lignes max est atteint, remise  zro de vcs
			vcs <= "00000000000";
			else
			vcs <= vcs + 1; --incrmentation du compteur vertical
		end if;
	end if;
end process;

process(raz_int, rd_fifo, cpt_fifo_en)
		begin	
		
			fifo1_rd_en <= '0';
			fifo2_rd_en <= '0';
			
		if raz_int ='1' then 
			cpt_fifo_en <= "11";	
			
		elsif(rd_fifo'event and rd_fifo = '1')then
				if cpt_fifo_en = "11" then 
				cpt_fifo_en <= "00";
				else
				cpt_fifo_en <= cpt_fifo_en + 1;		
		end if;	
		end if;
fifo1_rd_en <= cpt_fifo_en(0) nand cpt_fifo_en(1);
fifo2_rd_en <= cpt_fifo_en(0) or cpt_fifo_en(1);

fifo1_rd_en_in <= cpt_fifo_en(0) nand cpt_fifo_en(1);
fifo2_rd_en_in <= cpt_fifo_en(0) or cpt_fifo_en(1);				
		
end process; 
   
sync_int <= '0' when ((hcs < 63) or (((vcs < 1) and (hcs < 500)) or ((vcs = 409) and ((hcs < 1158) and (hcs > 596))))) else '1';

vidon <= '1' when ((((hcs < hfp) and (hcs > hbp)) and ((vcs <= vfpf1) and (vcs >= vbpf1))) or (((hcs < hfp) and (hcs > hbp)) and ((vcs <= vfpf2) and (vcs >= vbpf2)))) else '0'; 

rd_fifo <= '1' when ((((hcs < hfp) and (hcs > hbp)) and ((vcs <= vfpf1) and (vcs >= vbpf1))) or (((hcs < hfp) and (hcs > hbp)) and ((vcs <= vfpf2) and (vcs >= vbpf2)))) else '0'; 
end mire_819_lignes;












