library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity convertisseur_top_module is


Port (
		Video_IN_TDA8708 : in  STD_LOGIC_VECTOR (7 downto 0);
		Clk_TDA8708 : out  STD_LOGIC;
		Sel_TDA8708 : out  STD_LOGIC;
		GATE_A_TDA8708 : out std_logic;
		GATE_B_TDA8708 : out std_logic;
		VS_LM1881 : in  STD_LOGIC;
		HS_LM1881 : in  STD_LOGIC;
		BURST_BLANKING_LM1881 : in  STD_LOGIC;
		Video_out_TDA8702 : out  STD_LOGIC_VECTOR (7 downto 0);
		Clk_TDA8702 : out  STD_LOGIC;
		Master_raz : in  STD_LOGIC;
		mclk : in std_logic;
		
		START_819_out : out std_logic;
		RST_FIFO_out : out std_logic;
		VIDON_625_out : out std_logic;
		wr_clk_out : out std_logic;
		fifo1_rd_en_out : out std_logic;
		fifo2_rd_en_out : out std_logic;
		sel_in_out : out std_logic_vector( 1 downto 0);
		rd_clk_out : out std_logic;
		vidon_819_out :out std_logic;
		sync_int_out : out std_logic
		--video_819_fifo1_out : out std_logic_vector( 7 downto 0);
		--video_819_fifo2_out : out std_logic_vector( 7 downto 0)
		);		
		
end convertisseur_top_module;

architecture Behavioral of convertisseur_top_module is

signal START_819_INT : std_logic;--
signal RST_FIFO_INT : std_logic;
signal VIDON_625_int : std_logic;
signal wr_clk_int : std_logic;
signal fifo1_rd_en, fifo2_rd_en : std_logic;
signal sel_in : std_logic_vector( 1 downto 0);
signal rd_clk_int : std_logic;
signal vidon_819 : std_logic;
signal sync_int : std_logic;
signal video_819_fifo1 : std_logic_vector( 7 downto 0);
signal video_819_fifo2 : std_logic_vector( 7 downto 0);

		
component driver_625_lignes is
		port
		( 
		mclk : in STD_LOGIC; -- signal d'horloge interne principal cadenc  50MHz
		raz : in STD_LOGIC;-- remise  zro externe par circuit RC
		hsync : in STD_LOGIC;-- signal de synchronisation horizontale venant du LM1881
		vsync : in STD_LOGIC;-- signal de synchronisation verticale venant du LM1881
		BURST_BKG : in STD_LOGIC;-- signal de burst blanking venant du LM1881
		GATE_A : out STD_LOGIC;-- signal de commande du TDA8708A
		GATE_B : out STD_LOGIC;-- signal de commande du TDA8708A
		Sel_TDA8708A : out STD_LOGIC;-- signal d'activation de l'entre vido du TDA8708A
		FIFO_WE : out STD_LOGIC;-- signal d'activation en criture des fifos
		pixel_clk : out STD_LOGIC;-- horloge pixel pour TDA8708A et FIFOs
		clk_TDA8708 : out STD_LOGIC;-- horloge pixel pour TDA8708A et FIFOs
		read_process_en : out std_logic; -- signal de dmarrage du process 819 lignes	
		RST_FIFO : out STD_LOGIC-- signal de remise  zro des FIFOs actif  chaque trame
		);
end component;



component fifo2 is
	port
	(
    rst       : in std_logic;--reset
    wr_en     : in std_logic;--criture autorise
    wr_clk    : in std_logic;-- horloge d'criture
    wr_data   : in std_logic_vector( 7 downto 0 );-- donne d'entre=>8 bits
    rd_en     : in std_logic;-- lecture autorise
    rd_clk    : in std_logic;-- horloge d'criture
    rd_data   : out std_logic_vector( 7 downto 0 )-- donne de sortie => 8bits
    );

end component;


component interpolateur_8_bits is
Port ( 
		data_fifo_1 : in  STD_LOGIC_VECTOR (7 downto 0);
      data_fifo_2 : in  STD_LOGIC_VECTOR (7 downto 0);
      sel_fifo : in  STD_LOGIC_VECTOR (1 downto 0);
	 	sync : in  STD_LOGIC;
	 	vidon : in  STD_LOGIC;
      interpolateur_data_out : out  STD_LOGIC_VECTOR (7 downto 0)
		);
end component;

component mire_819_lignes is

port( 
		mclk : in STD_LOGIC; -- signal d'horloge principal cadenc  50MHz
		raz : in STD_LOGIC;-- remise  zro hardware
		rd_process_en : in std_logic;-- signal venant du bloc 625
		SYNC_INT : out STD_LOGIC;-- signal de synchronisation verticale
		vidon : out STD_LOGIC;-- signal d'activation de la vido allant  l'interpolateur
		fifo1_rd_en : out std_logic;-- signal d'activation de la fifo1
		fifo2_rd_en : out std_logic;-- signal d'activation de la fifo2
		sel_in : out std_logic_vector(1 downto 0);-- signal de fonctionnement pour l'interpolateur
		rd_clock_ext : out std_logic;-- horloge pixel 819 lignes pour le TDA8702
		rd_clock_int : out std_logic-- horloge pixel 819 lignes pour les FIFOs
		);
end component;

component fifo is
	port
	(
    rst       : in std_logic;--reset
    wr_en     : in std_logic;--criture autorise
    wr_clk    : in std_logic;-- horloge d'criture
    wr_data   : in std_logic_vector( 7 downto 0 );-- donne d'entre=>8 bits
    rd_en     : in std_logic;-- lecture autorise
    rd_clk    : in std_logic;-- horloge d'criture
    rd_data   : out std_logic_vector( 7 downto 0 )-- donne de sortie => 8bits
    );

end component;

begin


		START_819_out <= START_819_INT;
		RST_FIFO_out <= RST_FIFO_INT;
		VIDON_625_out <= VIDON_625_int;
		wr_clk_out <= wr_clk_int;
		fifo1_rd_en_out <= fifo1_rd_en;
		fifo2_rd_en_out <= fifo2_rd_en;
		sel_in_out <= sel_in;
		rd_clk_out <= rd_clk_int;
		vidon_819_out <= vidon_819;
		sync_int_out <= sync_int;

u1 : driver_625_lignes
	port map( mclk => mclk,
				raz => master_raz,
				hsync => hs_lm1881,
				vsync => vs_lm1881,
				burst_bkg => burst_blanking_lm1881,
				gate_a => gate_a_tda8708,
				gate_b => gate_b_tda8708,
				sel_tda8708a => sel_tda8708,
				fifo_we => vidon_625_int,
				pixel_clk => wr_clk_int,
				clk_TDA8708 => clk_TDA8708,
				rst_fifo => rst_fifo_int,
				read_process_en => start_819_int
	);
	
u2 : fifo
	port map(rst => rst_fifo_int,
				wr_data => Video_IN_TDA8708,
				wr_clk => wr_clk_int,
				wr_en => vidon_625_int,
				rd_data => video_819_fifo1,
				rd_en => fifo1_rd_en,
				rd_clk => rd_clk_int
				);
	
u3 : interpolateur_8_bits
	port map(interpolateur_data_out => Video_out_TDA8702,
				data_fifo_1 => video_819_fifo1,
				data_fifo_2 => video_819_fifo2,
				sel_fifo => sel_in,
				sync => sync_int,
				vidon => vidon_819	
				);
	
u4 : mire_819_lignes
	port map(mclk => mclk,
				raz => master_raz,
				rd_process_en => start_819_int, 
				SYNC_INT => sync_int,
				vidon => vidon_819,
				fifo1_rd_en => fifo1_rd_en,
				fifo2_rd_en => fifo2_rd_en,
				rd_clock_ext => Clk_TDA8702,
				rd_clock_int => rd_clk_int,
				sel_in => sel_in
				);
	
	
u5 : fifo2
	port map(rst =>rst_fifo_int,
				wr_data => Video_IN_TDA8708,
				wr_clk => wr_clk_int,
				wr_en => vidon_625_int,
				rd_data => video_819_fifo2,
				rd_en => fifo2_rd_en,
				rd_clk => rd_clk_int
				);
	
	


end Behavioral;

